<?php // $id$

function mb_get_grades($uid = false) {
    global $CFG;
    require_once($CFG->dirroot.'/grade/lib.php');

    $grades = @grade_get_formatted_grades();
    $grades = $grades[0];

    if (!is_array($grades)) {
        return false;
    }

    /// Calculate projected grade based on work completed so far, before we do anything else.
    foreach ($grades as $sid => $allinfo) {
        $numcomplete = 0;
        $gradetot = 0;
        foreach ($allinfo['uncategorised'] as $gname => $ginfo) {
            if (($gname == 'stats') || !is_numeric($ginfo['grade']) || ($ginfo['grade'] < 0)) {
                continue;
            }
            $gradetot += $ginfo['percent'];
            $numcomplete++;
        }
        if ($numcomplete > 0) {
            $grades[$sid]['student_data']['projectedgrade'] = (float)$gradetot / (float)$numcomplete;
        } else {
            $grades[$sid]['student_data']['projectedgrade'] = '-';
        }
    }

    if ($uid) {
        return $grades[$uid];
    } else {
        return $grades;
    }
}

function mb_get_class_position(&$grades, $uid, $group=false, $courseid=false) {

    /// If a group has been requested and not been provided, look it up (this needs the course id).
    /// Otherwise assume that the group parameter is the group id.
    if (($group !== false) && !is_numeric($group) && ($courseid !== false)) {
        if ($group = user_group($courseid, $uid)) {
            $group = $group->id;
        } else {
            $group = false;
        }
    }

    /// If there is a group request, but there are no students in that group, cancel the group request.
    if (($group !== false) && !($groupstudents = get_group_students($group))) {
        $group = false;
    }

    if (empty($grades)) {
        return array(0,0);
    }
    
    $sgrades = array();
    $scount = 0;
    /// Extract just the grades by student, and optionally ones in the group.
    foreach ($grades as $sid => $graderec) {
        if (($group === false) || (isset($groupstudents[$sid]))) {
            if (is_numeric($graderec['student_data']['projectedgrade'])) {
                $sgrades[$sid] = $graderec['student_data']['projectedgrade'];
            } else {
                $sgrades[$sid] = 0;
            }
            $scount++;
        }
    }
    /// Sort the grades by student, highest to lowest.
    arsort($sgrades);
    /// Create a new, numeric indexed array of this order.
    $studs = array_keys($sgrades);
    /// Extract the numeric position of the requested student (add one for zero based).
    $pos = reset(array_keys($studs, $uid)) + 1;
    return array($pos, $scount);
}

function mb_get_group_average(&$grades, $uid, $group=true, $courseid=false) {

    /// If a group id not been provided, look it up (this needs the course id).
    /// Otherwise assume that the group parameter is the group id.
    if ($group === true) {
        if (($courseid === false) || !($group = user_group($courseid, $uid))) {
            return 0;
        }
        $group = $group->id;
    }

    /// If there are no students in that group, cancel the group request.
    if ($group && !($groupstudents = get_group_students($group))) {
        return 0;
    }

    if (empty($grades)) {
        return '';
    }
    
    $sgrades = 0;
    $scount = 0;
    /// Extract just the grades by student, and optionally ones in the group.
    foreach ($grades as $sid => $graderec) {
        if (!$group || isset($groupstudents[$sid])) {
            if (is_numeric($graderec['student_data']['projectedgrade']) && 
                ($graderec['student_data']['projectedgrade'] != '-')) {
                $sgrades += $graderec['student_data']['projectedgrade'];
                $scount++;
            }
        }
    }

    return sprintf("%.2f", ($sgrades / $scount));
}

function mb_display_student_stats(&$course, &$user, $tid) {
    $grades = mb_get_grades();
    $userstats = $grades[$user->id]['student_data'];
    $mygroupid = mygroupid($course->id);

    $classpos = mb_get_class_position($grades, $user->id);
    if ($mygroupid > 0) {
        $grouppos = mb_get_class_position($grades, $user->id, $mygroupid);
    }

    $targetsub = get_record('assignment_submissions', 'assignment', $tid, 'userid', $user->id);

    echo '<div class="heading" align="center">'.$course->fullname.' Course Statistics for '.fullname($user).'</div>';
    $table = new Object();
    $table->cellpadding = 8;
    $table->head = array('','','');
    $table->wrap = array(true, true, true);

    $table->data[] = array('<b>'.get_string('generalstats', 'block_marking').'</b>', '', '');
    $table->data[] = array('', '', '');

    if ($targetsub !== false) {
        $targetgrade = $targetsub->grade;
        $targetlettero = grade_get_grade_letter($course->id, $targetgrade);
        $targetgrade .= '%';
        $targetletter = $targetlettero->letter;
        $targetdev = $userstats['projectedgrade'] - $targetgrade;
        $targetdevlettero = grade_get_grade_letter($course->id, $targetdev);
        $targetdev .= '%';
        $targetdevletter = $targetdevlettero->letter;
    } else {
        $targetgrade = get_string('notargetdefined', 'block_marking');
        $targetletter = '';
        $targetdev = 0;
        $targetdevletter = '';
    } 
    $table->data[] = array(get_string('target', 'block_marking'), $targetgrade, $targetletter);
    $gradeletter = grade_get_grade_letter($course->id, $userstats['projectedgrade']);
    $table->data[] = array(get_string('courseaverage2', 'block_marking'),
                           get_string('courseaverage2a', 'block_marking', $userstats['projectedgrade']),
                           $gradeletter->letter);
    $table->data[] = array(get_string('deviation', 'block_marking'), $targetdev, $targetdevletter);

    $table->data[] = array('', '', '');

    $a->pos = $classpos[0];
    $a->num = $classpos[1];
    $table->data[] = array(get_string('courseposition2', 'block_marking'),
                           get_string('courseposition2a', 'block_marking', $a), '');
    if ($mygroupid > 0) {
        $a->pos = $grouppos[0];
        $a->num = $grouppos[1];
        $table->data[] = array(get_string('groupposition2', 'block_marking'),
                               get_string('groupposition2a', 'block_marking', $a), '');
    }

    $table->data[] = array('', '', '');

    $grade = mb_get_group_average($grades, $user->id, false);
    $gradeletter = grade_get_grade_letter($course->id, $grade);
    $table->data[] = array(get_string('classaverage2', 'block_marking'),
                           get_string('classaverage2a', 'block_marking', $grade), $gradeletter->letter);
    $grade = mb_get_group_average($grades, $user->id, $mygroupid);
    $gradeletter = grade_get_grade_letter($course->id, $grade);
    if ($mygroupid > 0) {
        $table->data[] = array(get_string('groupaverage2', 'block_marking'),
                               get_string('groupaverage2a', 'block_marking', $grade), $gradeletter->letter);
    }

    mb_get_module_stats($table, $course, $user);

    print_table($table);
}

function mb_display_teacher_stats(&$course, $tid, $userid=0) {
    global $CFG;
    require_once($CFG->libdir.'/tablelib.php');

    echo '<div class="heading" align="center">'.$course->fullname.' Course Statistics</div>';

    if ($userid) {
        $user = get_record('user', 'id', $userid);
        return mb_display_student_stats($course, $user, $tid);
    }

/// Check to see if groups are being used in this assignment
    $changegroup = optional_param('group', -1, PARAM_INT);
    $page = optional_param('page', 0, PARAM_INT);
    $perpage = optional_param('perpage', 20, PARAM_INT);
    $baseurl = 'stats.php?id='.$course->id.'&amp;tid='.$tid.'&amp;page='.$page.'&amp;perpage='.$perpage;

    $groupmode = groupmode($course);
    $currentgroup = get_and_set_current_group($course, $groupmode, $changegroup);
    if ($groupmode) {   // Groups are being used
        setup_and_print_groups($course, $groupmode, $baseurl);
        echo '<br />';
    }

    $baseurl = 'stats.php?id='.$course->id.'&amp;tid='.$tid.'&amp;group='.$currentgroup;

    /// Flexible tables taken from 'user/index.php'.
    $tablecolumns = array('picture', 'fullname', 'grouppos', 'classpos', 'courseaverage', 'groupaverage', 
                          'classaverage');
    $tableheaders = array('', get_string('fullname'), get_string('groupposition2', 'block_marking'), 
                          get_string('courseposition2', 'block_marking'), get_string('courseaverage2', 'block_marking'),
                          get_string('groupaverage2', 'block_marking'), get_string('classaverage2', 'block_marking'));

    $table = new flexible_table('marking-stats-students');

    $table->define_columns($tablecolumns);
    $table->define_headers($tableheaders);
    $table->define_baseurl($baseurl);

    $table->sortable(false);

    $table->set_attribute('cellspacing', '0');
    $table->set_attribute('cellpadding', '6');
    $table->set_attribute('id', 'students');
    $table->set_attribute('class', 'generaltable generalbox');

    $table->set_control_variables(array(
        TABLE_VAR_SORT    => 'ssort',
        TABLE_VAR_HIDE    => 'shide',
        TABLE_VAR_SHOW    => 'sshow',
        TABLE_VAR_IFIRST  => 'sifirst',
        TABLE_VAR_ILAST   => 'silast',
        TABLE_VAR_PAGE    => 'spage'
    ));
    $table->setup();

    if(SITEID == $course->id) {
        $select = 'SELECT u.id, u.username, u.firstname, u.lastname, u.email, u.city, u.country, 
                      u.picture, u.lang, u.timezone, u.emailstop, u.maildisplay, u.lastaccess ';
        $from   = 'FROM '.$CFG->prefix.'user u ';
        $where  = 'WHERE confirmed = 1 AND u.deleted = 0 ';
    }
    else {
        $select = 'SELECT u.id, u.username, u.firstname, u.lastname, u.email, u.city, u.country, 
                      u.picture, u.lang, u.timezone, u.emailstop, u.maildisplay, s.timeaccess AS lastaccess ';
        $from   = 'FROM '.$CFG->prefix.'user u LEFT JOIN '.$CFG->prefix.'user_students s ON s.userid = u.id ';
        $where  = 'WHERE s.course = '.$course->id.' AND u.deleted = 0 ';
    }

    if ($currentgroup) {    // Displaying a group by choice
        // FIX: TODO: This will not work if $currentgroup == 0, i.e. "those not in a group"
        $from  .= 'LEFT JOIN '.$CFG->prefix.'groups_members gm ON u.id = gm.userid ';
        $where .= ' AND gm.groupid = '.$currentgroup;
    }

//    if($course->id == SITEID) {
//        $where .= ' AND u.id NOT IN ('.implode(',', $exceptions).')';
//    }

    $totalcount = count_records_sql('SELECT COUNT(*) '.$from.$where);

    if($table->get_sql_where()) {
        $where .= ' AND '.$table->get_sql_where();
    }

    if($table->get_sql_sort()) {
        $sort = ' ORDER BY '.$table->get_sql_sort();
    }
    else {
        $sort = '';
    }

    $matchcount = count_records_sql('SELECT COUNT(*) '.$from.$where);

    $table->initialbars($totalcount > $perpage);
    $table->pagesize($perpage, $matchcount);

    if($table->get_page_start() !== '' && $table->get_page_size() !== '') {
        $limit = ' '.sql_paging_limit($table->get_page_start(), $table->get_page_size());
    }
    else {
        $limit = '';
    }    
    
    $students = get_records_sql($select.$from.$where.$sort.$limit);

    if (!empty($students))  {
        $grades = mb_get_grades();
        foreach ($students as $student) {
            $userstats = $grades[$student->id]['student_data'];
            if ($currentgroup <= 0) {
                $mygroup = user_group($course->id, $student->id);
                $mygroupid = $mygroup->id;
            } else {
                $mygroupid = $currentgroup;
            }

            $classpos = mb_get_class_position($grades, $student->id);
            if ($mygroupid > 0) {
                $grouppos = mb_get_class_position($grades, $student->id, $mygroupid);
            } else {
                $grouppos = array(0,0);
            }
            $classavg = mb_get_group_average($grades, $student->id, false);
            $groupavg = mb_get_group_average($grades, $student->id, $mygroupid);

            $projectedgrade = ($userstats['projectedgrade'] != '-') ? 
                              sprintf('%.2f', $userstats['projectedgrade']) : '-';

            $table->add_data(array (
                    print_user_picture($student->id, $course->id, $student->picture, false, true),
                    '<strong><a href="'.$baseurl.'&amp;uid='.$student->id.'" title="Student Report">'.
                    fullname($student).'</a></strong>',$grouppos[0].'/'.$grouppos[1], 
                    $classpos[0].'/'.$classpos[1], $projectedgrade, $groupavg, $classavg));
        }
    }

    $table->print_html();

    if ($perpage == 99999) {
        echo '<div id="showall"><a href="'.$baseurl.'&amp;perpage='.DEFAULT_PAGE_SIZE.'">'.get_string('showperpage', '', DEFAULT_PAGE_SIZE).'</a></div>';
    }
    else if ($matchcount > 0 && $perpage < $matchcount) {
        echo '<div id="showall"><a href="'.$baseurl.'&amp;perpage=99999">'.get_string('showall', '', $matchcount).'</a></div>';
    }
}

function mb_get_module_stats(&$table, &$course, &$user) {
    global $CFG;

/// Array of functions to call for stats purposes for modules.
    $mod_stats_array = array(
        'assignment' => 1,
        'newassignment' => 1,
        'journal' => 1,
        'lesson' => 1,
        'forum' => 1,
        'quiz' => 1
        );

    /// Collect modules data
    get_all_mods($course->id, $mods, $modnames, $modnamesplural, $modnamesused);

    /// Put the mods into an array of modnames.
    $coursemods = array();
    foreach ($mods as $mod) {
        $coursemods[$mod->modname][$mod->instance] = $mod;
    }
    ksort($coursemods);
    unset($mods);

    $statstring = get_string('stats', 'block_marking');
    foreach ($coursemods as $modname => $mods) {
        /// Look to see if this is a gradeable function.
        $libfile = $CFG->dirroot.'/mod/'.$modname.'/lib.php';
        if (file_exists($libfile)) {
            require_once($libfile);
            /// Use the first entry for testing.
            $mod = current($mods);
            if ($mod->visible &&                    // Skip hidden modules
                isset($mod_stats_array[$modname])) {

                $statsfunction = 'mb_'.$modname.'_stats';
                if (function_exists($statsfunction)) {
                    $table->data[] = 'hr';
                    $table->data[] = array('<b>'.$mod->modfullname.' '.$statstring.'</b>', '', '');
                    $lines = $statsfunction($mods, $user);
                    foreach ($lines as $line) {
                        $table->data[] = $line;
                    }
                }
            }
        }
    }
}

function mb_assignment_stats(&$mods, &$user) {
    $lines = array();

    /// Collect all the assignment id's so we can get them all in one db call.
    $select = '';
    $sselect = '';
    foreach ($mods as $mod) {
        if (empty($select)) {
            $select = 'id IN ('.$mod->instance;
            $sselect = 'assignment IN ('.$mod->instance;
        } else {
            $select .= ', '.$mod->instance;
            $sselect .= ', '.$mod->instance;
        }
    }
    if (!empty($select)) {
        $select .= ')';
        $sselect .= ') AND userid = '.$user->id;
        if (!($assignments = get_records_select('assignment', $select))) {
            $assignments = array();
        }
        if (!($submissions = get_records_select('assignment_submissions', $sselect, 'assignment', 
                                                'assignment,id,userid,timecreated,timemodified,timemarked'))) {
            $submissions = array();
        }
    } else {
        $assignments = array();
        $submissions = array();
    }

    $earlysubmission = new Object();
    $earlysubmission->name = get_string('earlysubmissions', 'block_marking');
    $earlysubmission->count = 0;
    $earlysubmission->timediff = 0;

    $outstandingsubmission = new Object();
    $outstandingsubmission->name = get_string('outstandingsubmissions', 'block_marking');
    $outstandingsubmission->count = 0;
    $outstandingsubmission->timediff = 0;

    $latesubmission = new Object();
    $latesubmission->name = get_string('latesubmissions', 'block_marking');
    $latesubmission->count = 0;
    $latesubmission->timediff = 0;

    $timenow = time();

    foreach ($assignments as $assid => $assignment) {
        if (isset($submissions[$assid]) && ($submissions[$assid]->timemodified > 0) &&
            ($submissions[$assid]->timemodified < $assignment->timedue)) {
            $earlysubmission->count++;
            $earlysubmission->timediff += ($assignment->timedue - $submissions[$assid]->timemodified);
        } else if (isset($submissions[$assid]) && ($submissions[$assid]->timemodified > 0) &&
                   ($assignment->timedue > 0)) {
            $latesubmission->count++;
            $latesubmission->timediff += ($submissions[$assid]->timemodified - $assignment->timedue);
        } else if (!isset($submissions[$assid]) || ($submissions[$assid]->timemodified <= 0)){
            $outstandingsubmission->count++;
            if ($assignment->timedue > 0) {
                $timediff = $timenow - $assignment->timedue;
                $outstandingsubmission->timediff =
                    $timediff > $outstandingsubmission->timediff ? $timediff : $outstandingsubmission->timediff;
            }
        }
    }

    $earlysubmission->averagestring = mb_get_average_time_string($earlysubmission, 'earlyaverage');
    $latesubmission->averagestring = mb_get_average_time_string($latesubmission, 'lateaverage');
    $outstandingsubmission->latestring = mb_get_late_time_string($outstandingsubmission, 'latest');

    $lines[] = array($earlysubmission->name, $earlysubmission->count, $earlysubmission->averagestring);
    $lines[] = array($outstandingsubmission->name, $outstandingsubmission->count,
                     $outstandingsubmission->latestring);
    $lines[] = array($latesubmission->name, $latesubmission->count, $latesubmission->averagestring);

    return $lines;
}

function mb_newassignment_stats(&$mods, &$user) {
    $lines = array();

    /// Collect all the assignment id's so we can get them all in one db call.
    $select = '';
    $sselect = '';
    foreach ($mods as $mod) {
        if (empty($select)) {
            $select = 'id IN ('.$mod->instance;
            $sselect = 'newassignment IN ('.$mod->instance;
        } else {
            $select .= ', '.$mod->instance;
            $sselect .= ', '.$mod->instance;
        }
    }
    if (!empty($select)) {
        $select .= ')';
        $sselect .= ') AND userid = '.$user->id;
        if (!($assignments = get_records_select('newassignment', $select))) {
            $assignments = array();
        }
        if (!($submissions = get_records_select('newassignment_submissions', $sselect, 'newassignment', 
                                                'newassignment,id,userid,timecreated,timemodified,timemarked'))) {
            $submissions = array();
        }
    } else {
        $assignments = array();
        $submissions = array();
    }

    $earlysubmission = new Object();
    $earlysubmission->name = get_string('earlysubmissions', 'block_marking');
    $earlysubmission->count = 0;
    $earlysubmission->timediff = 0;

    $outstandingsubmission = new Object();
    $outstandingsubmission->name = get_string('outstandingsubmissions', 'block_marking');
    $outstandingsubmission->count = 0;
    $outstandingsubmission->timediff = 0;

    $latesubmission = new Object();
    $latesubmission->name = get_string('latesubmissions', 'block_marking');
    $latesubmission->count = 0;
    $latesubmission->timediff = 0;

    $timenow = time();

    foreach ($assignments as $assid => $assignment) {
        if (isset($submissions[$assid]) && ($submissions[$assid]->timemodified > 0) &&
                 ($submissions[$assid]->timemodified < $assignment->timedue)) {
            $earlysubmission->count++;
            $earlysubmission->timediff += ($assignment->timedue - $submissions[$assid]->timemodified);
        } else if (isset($submissions[$assid]) && ($submissions[$assid]->timemodified > 0) &&
                   ($assignment->timedue > 0)) {
            $latesubmission->count++;
            $latesubmission->timediff += ($submissions[$assid]->timemodified - $assignment->timedue);
        } else if (!isset($submissions[$assid]) || ($submissions[$assid]->timemodified <= 0)){
            $outstandingsubmission->count++;
            if ($assignment->timedue > 0) {
                $timediff = $timenow - $assignment->timedue;
                $outstandingsubmission->timediff =
                    $timediff > $outstandingsubmission->timediff ? $timediff : $outstandingsubmission->timediff;
            }
        }
    }

    $earlysubmission->averagestring = mb_get_average_time_string($earlysubmission, 'earlyaverage');
    $latesubmission->averagestring = mb_get_average_time_string($latesubmission, 'lateaverage');
    $outstandingsubmission->latestring = mb_get_late_time_string($outstandingsubmission, 'latest');

    $lines[] = array($earlysubmission->name, $earlysubmission->count, $earlysubmission->averagestring);
    $lines[] = array($outstandingsubmission->name, $outstandingsubmission->count,
                     $outstandingsubmission->latestring);
    $lines[] = array($latesubmission->name, $latesubmission->count, $latesubmission->averagestring);

    return $lines;
}

function mb_lesson_stats(&$mods, &$user) {
    $lines = array();

    /// Collect all the assignment id's so we can get them all in one db call.
    $select = '';
    $sselect = '';
    foreach ($mods as $mod) {
        if (empty($select)) {
            $select = 'id IN ('.$mod->instance;
            $sselect = 'lesson IN ('.$mod->instance;
        } else {
            $select .= ', '.$mod->instance;
            $sselect .= ', '.$mod->instance;
        }
    }
    if (!empty($select)) {
        $select .= ')';
        $sselect .= ') AND userid = '.$user->id;
        if (!($assignments = get_records_select('lesson', $select))) {
            $assignments = array();
        }
        if (!($submissions = get_records_select('lesson_grades', $sselect, 'lessonid', 
                                                'lessonid,id,userid,completed'))) {
            $submissions = array();
        }
    } else {
        $assignments = array();
        $submissions = array();
    }

    $earlysubmission = new Object();
    $earlysubmission->name = get_string('earlysubmissions', 'block_marking');
    $earlysubmission->count = 0;
    $earlysubmission->timediff = 0;

    $outstandingsubmission = new Object();
    $outstandingsubmission->name = get_string('outstandingsubmissions', 'block_marking');
    $outstandingsubmission->count = 0;
    $outstandingsubmission->timediff = 0;

    $latesubmission = new Object();
    $latesubmission->name = get_string('latesubmissions', 'block_marking');
    $latesubmission->count = 0;
    $latesubmission->timediff = 0;

    $timenow = time();

    foreach ($assignments as $assid => $assignment) {
        if (isset($submissions[$assid]) && ($submissions[$assid]->completed < $assignment->deadline)) {
            $earlysubmission->count++;
            $earlysubmission->timediff += ($assignment->deadline - $submissions[$assid]->completed);
        } else if (isset($submissions[$assid]) && ($assignment->deadline > 0)) {
            $latesubmission->count++;
            $latesubmission->timediff += ($submissions[$assid]->completed - $assignment->deadline);
        } else if (!isset($submissions[$assid])){
            $outstandingsubmission->count++;
            if ($assignment->deadline > 0) {
                $timediff = $timenow - $assignment->deadline;
                $outstandingsubmission->timediff =
                    $timediff > $outstandingsubmission->timediff ? $timediff : $outstandingsubmission->timediff;
            }
        }
    }

    $earlysubmission->averagestring = mb_get_average_time_string($earlysubmission, 'earlyaverage');
    $latesubmission->averagestring = mb_get_average_time_string($latesubmission, 'lateaverage');
    $outstandingsubmission->latestring = mb_get_late_time_string($outstandingsubmission, 'latest');

    $lines[] = array($earlysubmission->name, $earlysubmission->count, $earlysubmission->averagestring);
    $lines[] = array($outstandingsubmission->name, $outstandingsubmission->count,
                     $outstandingsubmission->latestring);
    $lines[] = array($latesubmission->name, $latesubmission->count, $latesubmission->averagestring);

    return $lines;
}

function mb_quiz_stats(&$mods, &$user) {
    $lines = array();

    /// Collect all the assignment id's so we can get them all in one db call.
    $select = '';
    $sselect = '';
    foreach ($mods as $mod) {
        if (empty($select)) {
            $select = 'id IN ('.$mod->instance;
            $sselect = 'lesson IN ('.$mod->instance;
        } else {
            $select .= ', '.$mod->instance;
            $sselect .= ', '.$mod->instance;
        }
    }
    if (!empty($select)) {
        $select .= ')';
        $sselect .= ') AND userid = '.$user->id;
        if (!($assignments = get_records_select('quiz', $select))) {
            $assignments = array();
        }
        if (!($submissions = get_records_select('quiz_grades', $sselect, 'quiz', 
                                                'quiz,id,userid,timemodified'))) {
            $submissions = array();
        }
    } else {
        $assignments = array();
        $submissions = array();
    }

    $earlysubmission = new Object();
    $earlysubmission->name = get_string('earlysubmissions', 'block_marking');
    $earlysubmission->count = 0;
    $earlysubmission->timediff = 0;

    $outstandingsubmission = new Object();
    $outstandingsubmission->name = get_string('outstandingsubmissions', 'block_marking');
    $outstandingsubmission->count = 0;
    $outstandingsubmission->timediff = 0;

    $latesubmission = new Object();
    $latesubmission->name = get_string('latesubmissions', 'block_marking');
    $latesubmission->count = 0;
    $latesubmission->timediff = 0;

    $timenow = time();

    foreach ($assignments as $assid => $assignment) {
        if (isset($submissions[$assid]) && ($submissions[$assid]->timemodified < $assignment->timeclose)) {
            $earlysubmission->count++;
            $earlysubmission->timediff += ($assignment->timeclose - $submissions[$assid]->timemodified);
        } else if (isset($submissions[$assid]) && ($assignment->timeclose > 0)) {
            $latesubmission->count++;
            $latesubmission->timediff += ($submissions[$assid]->timemodified - $assignment->timeclose);
        } else if (!isset($submissions[$assid])){
            $outstandingsubmission->count++;
            if ($assignment->timeclose > 0) {
                $timediff = $timenow - $assignment->timeclose;
                $outstandingsubmission->timediff =
                    $timediff > $outstandingsubmission->timediff ? $timediff : $outstandingsubmission->timediff;
            }
        }
    }

    $earlysubmission->averagestring = mb_get_average_time_string($earlysubmission, 'earlyaverage');
    $latesubmission->averagestring = mb_get_average_time_string($latesubmission, 'lateaverage');
    $outstandingsubmission->latestring = mb_get_late_time_string($outstandingsubmission, 'latest');

    $lines[] = array($earlysubmission->name, $earlysubmission->count, $earlysubmission->averagestring);
    $lines[] = array($outstandingsubmission->name, $outstandingsubmission->count,
                     $outstandingsubmission->latestring);
    $lines[] = array($latesubmission->name, $latesubmission->count, $latesubmission->averagestring);

    return $lines;
}

function mb_get_average_time_string($subrec, $string) {
    if ($subrec->count > 0) {
        $average = (int)($subrec->timediff / $subrec->count);
        $avg->days = (int)($average / (60*60*24));
        $avg->hours = (int)(($average - ($avg->days*60*60*24)) / (60*60));
        $avgstring = get_string($string, 'block_marking', $avg);
    } else {
        $avgstring = '';
    }
    return $avgstring;
}

function mb_get_late_time_string($subrec, $string) {
    if ($subrec->count > 0) {
        $avg->days = (int)($subrec->timediff / (60*60*24));
        $avgstring = get_string($string, 'block_marking', $avg);
    } else {
        $avgstring = '';
    }
    return $avgstring;
}
?>
